#!/usr/bin/env python
# -*- coding: utf-8 -*-

## \namespace servidor

import sys
import logging
import logging.config
import os
import commands
import subprocess
from subprocess import Popen
from subprocess import call as subcall

from threading import Lock, Thread
import threading
from socket import socket, error, SHUT_RDWR, AF_INET, SOCK_STREAM

from PyQt4 import QtGui, QtCore
from PyQt4.QtCore import QThread, SIGNAL

import time

from dbus.mainloop.qt import DBusQtMainLoop
from dbus.mainloop.glib import DBusGMainLoop

from hga.controlgrupos.servidor.serverprofesor import ServerProfesor
from hga.controlgrupos.servidor.sxmppserver import SXmppServer
from hga.gestioncontrolgrupos import GestionControlGrupos 
from hga.controlcompartir.servidor.shareddir import SSharedDir
from hga.controlgrupos.common.smdns import SMdns

from hga.controldesktop.controlpeticiones import parar_procesos_activos
from hga.controlstreaming.servidor.controlstreaming import parar_vlc

from hga.controlgrupos.common.user_info import get_user_name 
from hga.controlgrupos.common.user_info import generate_jid
from hga.controlgrupos.common.user_info import get_hostname
from hga.controlgrupos.common.user_info import identify_user
from hga.controlgrupos.common.user_info import get_ip

from hga.controlgrupos.common.settings import VALID_CHARS
from hga.controlgrupos.common.settings import LOGGING_CONFIG_FILE
from hga.controlgrupos.common.settings import HGA_PID_FILE

from hga.controlgrupos.common.exception import SInvalidGroupName
from hga.controlgrupos.common.exception import NoDefaultDevice

from hga.controlgrupos.servidor.settingsservidor import CONSULTA_SERVER_COMMAND
from hga.controlgrupos.servidor.settingsservidor import KEY_SERVER

from hga.gui.common.sidentify import identifyWindow

logging.config.fileConfig(LOGGING_CONFIG_FILE)
linfo = logging.info
lerror = logging.error
ldebug = logging.debug
PORT_TCP = 6031
## \class AppCoreProfe
# \brief hgr teacher's core.
class AppCoreProfe():
    def __init__(self, gui, controlDesktop_handler, CEServer_handlers):
        # Diccionario con los datos de los usuarios. clave=JID
        self.users = dict()
        self.usersIps = dict()
        
        self.log = None
        self.jid = None
        self.hostname = None
        self.servidorActivo = False
        self.xmpp_server = None
        self.mdns = None
        self.flag = False

        self.shared=None
        self.shared_info = None

        self.xmpp_server = SXmppServer()

        # No instanciamos el bot aún, necesitamos los parámetros de conexión.
        self.bot = None
        self.sServidor = None
        self.muc_name = ""
        self.cambio_delay = 5
        # Lista para almacenar los grupos encontrados en la red
        self.listaGrupos = []
        self.resource_list = {}
        self.view_grupos()
        self.resource_list_lock = threading.Lock()  # Importa la biblioteca 'threading'
        self.ADDgrupo = None
        self.DELgrupo = None

        # Manejadores que se le pasan al bot.
        self.bot_handlers = {
            "NEW_USER_CB": self.anadir_alumno_lista,
            "DEL_USER_CB": self.borrar_alumno_lista,
            "ENVIAR_MENSAJE": self.sendmessage,
            "ACTUALIZA_CAPTURA": self.actualizarPantalla,
            "SEND_GUI_MSG": self.enviar_gui_msg,
            "STOP_PROGRESSBAR": self.stop_progressBar,
            "OFRECER_AYUDA_CLIENTE": self.ofrecer_ayuda_cliente,
            "IPS_USER":self.gestion_ips_cliente,

        }

        self._setup_logging()
        self.log.info(QtGui.QApplication.translate("AppCoreProfe.log", "Iniciando servidor."))

        if len(get_user_name()) == 0:
            ldebug("El username: '%s'" % get_user_name())
            identify_user()

        self.nick = get_user_name()

        self.controlDesktop_handler = controlDesktop_handler
        self.CEServer_handlers = CEServer_handlers
        self.gui = gui

        try:
            self.jid = generate_jid()
            self.hostname = get_hostname()
            
            self.xmpp_server.start()

            self._setup_mdns()
            
            self.servidorActivo = True
        except NoDefaultDevice, e:
            lerror("AppCoreProfe -> iniciar_servidor: No se ha podido obtener el host. %s" % e)
        except Exception, e:
            lerror("AppCoreProfe -> iniciar_servidor: %s" % e)

    def iniciar_servidor(self):
        if self.gui.procesos_progressBar == None:
            try:
                self.jid = generate_jid()
                self.hostname = get_hostname()
                self.start_progressBar(KEY_SERVER,CONSULTA_SERVER_COMMAND)
                self.startServer_thread = StartServer_thread(self.xmpp_server)
                self.gui.connect(self.startServer_thread, QtCore.SIGNAL("finished()"), self.finalizar_thread)
                self.enviar_gui_msg(u"Conexión detectada. Se abrirá el grupo una vez iniciado el servicio.","information")
                self.startServer_thread.start()
            except NoDefaultDevice, e:
                self.enviar_gui_msg(u"Parece que no hay conexión. Vuelva a intentarlo.","information")
                lerror("AppCoreProfe -> iniciar_servidor: %s" % e)
            except Exception, e:
                lerror("AppCoreProfe -> iniciar_servidor: %s" % e)
        else:
            self.enviar_gui_msg(u"Se está ejecutando otro proceso.","information")

    def finalizar_thread(self):
        self.stop_progressBar()
        if self.startServer_thread.estado:
            try:
                self._setup_mdns()
                self.servidorActivo = True
                self.crear_recurso(self.muc_name)
            except Exception, e:
                lerror("AppCoreProfe -> iniciar_servidor: %s" % e)
                self.servidorActivo = False
        else:
            self.servidorActivo = False

    def sanitize_muc_name(self, data):
        # Restricts the MUC name to ASCII only characters.
        data = data.replace(' ', '_')
        for a,b in ( ('á', 'a'), ('é', 'e'), ('í', 'i'), ('ó', 'o'), ('ú', 'u'), ('ñ', 'n'),
                    ('Á', 'A'), ('É', 'E'), ('Í', 'I'), ('Ó', 'O'), ('Ú', 'U'), ('Ñ', 'N') ):
            data=data.replace(a, b)
        data = str(filter(lambda k: k in VALID_CHARS, data)).lower()
        return data

    ## \fn crear_recurso(self, muc_name)
    # \brief Called then the user click on the Create resource button.
    # \param muc_name Name of the resource to be created.
    def crear_recurso(self, muc_name):
        muc_sane_name = self.sanitize_muc_name(muc_name)
        if len(muc_sane_name) == 0:
                self.log.error(QtGui.QApplication.translate("AppCoreProfe.log", "Nombre de grupo inválido: %1", 
                                                        None, QtGui.QApplication.UnicodeUTF8).arg(muc_name))
                raise SInvalidGroupName(muc_name)
        self.muc_name = muc_sane_name
        self.log.info(QtGui.QApplication.translate("AppCoreProfe.log", "Creando el grupo: %1", 
                                                    None, QtGui.QApplication.UnicodeUTF8).arg(muc_name))
        if self.muc_name in self.listaGrupos:
            self.log.error(
                QtGui.QApplication.translate(
                    "AppCoreProfe.log",
                    "Nombre de grupo inválido: {0}".format(self.muc_name),
                    None,
                )
            )
            msgBox = QtGui.QMessageBox()
            msgBox.setIcon(QtGui.QMessageBox.Information)
            msgBox.setText("Ya existe un grupo con ese nombre.")
            msgBox.setWindowTitle("Informacion")
            msgBox.exec_()
            return
        else:
            if not self.servidorActivo:
                self.iniciar_servidor()

            if self.servidorActivo:
                self.shared = SSharedDir(muc_sane_name)
                
                ldebug("Create resource '%s'\n%s\n" % (self.muc_name, '-'*80))
                self.muc_jid = self.generate_muc_jid(muc_sane_name)
                self.nick = get_user_name()
                self.bot = ServerProfesor( jid=self.jid,
                                    muc_jid=self.muc_jid,
                                    nick=self.nick,
                                    parent=self,
                                    )
                self.bot.set_core_handlers(self.bot_handlers)
                self.bot.vnc_handlers(self.controlDesktop_handler)
                self.bot.set_ce_handlers(self.CEServer_handlers)
                self.bot.connect(("127.0.0.1", 5222))
                self.bot.process(threaded=True)
                time.sleep(0.1)
                datos = {   'muc_jid': self.muc_jid,
                            'muc_name': self.muc_name,
                            'prof_name': self.nick,
                            'prof_jid': self.bot.fulljid,
                        }
                name = u"%s - %s" % (self.muc_name, self.nick)
                # Publishes the resource with mDNS.
                self.mdns.publish(name, datos)
                self.hiloSocket = HiloSocket(self)
                self.hiloSocket.start()
                self.log.info(QtGui.QApplication.translate("AppCoreProfe.log", "Grupo creado: %1 - %2 (%3)", 
                                                        None, QtGui.QApplication.UnicodeUTF8).arg(
                                                                    self.muc_name).arg(
                                                                    self.nick).arg(
                                                                    self.muc_jid)
                            )
                self.gui.emit(QtCore.SIGNAL('grupoNuevoCreado'), self.muc_name)
                
    ## \fn gestion_ips_cliente(self, nick, ipsCliente)
    # \brief Metodo encargado de guardar y gestionar las ips de los clientes que acceden al grupo.
    def gestion_ips_cliente(self, nick, ipsCliente):
        self.usersIps[nick]=ipsCliente

    ## \fn socketImagenes(self, nick)
    # \brief Metodo que abre el socket una vez creado el grupo y se queda a la espera.
    def socketImagenes(self):
        """Create a socket to receive images from the clients."""

        self.remove_proccess()
        self.create_socket_img()
        path_img = "/tmp/.hgr/Server/"
        if not os.path.exists(path_img):
            os.makedirs(path_img)
        self.socket_reciv_img(path_img=path_img)

    def create_socket_img(self):
        try:
            self.sServidor = socket(AF_INET, SOCK_STREAM)
            CONEXION = (get_ip(), PORT_TCP)
            # Escuchar peticiones en el puerto 6031.
            self.sServidor.bind(CONEXION)
            self.sServidor.listen(1)  # mínimo aceptaremos 30 conexiones
        except socket.timeout:
            lexp("Tiempo excedido al crear el socket")
        except OSError:
            lexp("Error al crear el socket de imagenes")
        except Exception as e:
            lerror("AppCoreProfe ->socketImagenes: error de socket: '%s'" % e)
            lexp("Error desconocido al crear el socket de imagenes")

    def remove_proccess(self):
        consulta = "nc -zv {} {}".format(get_ip(), PORT_TCP)
        resultPort = os.system(consulta)
        if resultPort == 0:
            os.system("fuser -k {}/tcp".format(PORT_TCP))

    def socket_reciv_img(self, path_img):
        """Socket to receive images from the clients
        and save them with the IP where they are sent.
        """
        while True:
            sck, addr = self.sServidor.accept()
            fichero = path_img + str(addr[0])
            # Recibimos la longitud que envía el cliente
            try:
                file = open(fichero, "wb")
                image_chunk = sck.recv(512)
                while image_chunk:
                    file.write(image_chunk)
                    image_chunk = sck.recv(512)
            except socket.timeout:
                lexp("Tiempo excedido al recibir la imagen")
            except OSError:
                lexp("Error al recibir la imagen")
            except Exception:
                lexp("Error desconocido al recibir la imagen del cliente")
            finally:
                file.close()
                sck.close()
    ## \fn actualizarPantalla(self, nick=None)
    # \brief Metodo que gestiona el mensaje del cliente una vez que envia por el socket su imagen de escritorio.
    # \param nick Nick del usuario que envia el mensaje y asi poder saber desde la ip que esta emitiendo.
    def actualizarPantalla(self, nick):
        ruta = '/tmp/.hgr/Server/'
        try:

            ipValida = self.users[nick].get('IP')
            ipSocket = None
            direcC = None

            ipsI = self.usersIps.get(nick)
            listIps = ipsI.split( )
            for i in listIps:
            	if i != ipValida:
               		ipSocket = i
                elif i == ipValida:
                	direcC = i
            if ipSocket in os.listdir(ruta):
             	os.rename(ruta+ipSocket, ruta+direcC)
        except KeyError, m:
            lerror("AppCoreProfe -> actualizarPantalla: error al obtener la ip: %s" % m)
        self.gui.emit(QtCore.SIGNAL('update_tabla_desktop'))

    def set_delay(self, delay):
        self.cambio_delay = delay
        try:
            usersTemp = self.users
            for user in usersTemp:
                if not usersTemp[user]["banned"]:
                    msg = "DELAY|"+str(self.cambio_delay)
                    if user in self.users:
                        self.sendmessage(usersTemp[user]['nick'],msg)
            usersTemp = None
        except RuntimeError as e:
            # La ejecución del bucle no finaliza por lo que se volverá a solicitar la función
            lerror("AppCoreProfe -> Cambio delay: def set_delay(delay): %s" % e)


    ## \fn generate_muc_jid(self)
    # \brief Generates a new MUC jid based on the resource name.
    def generate_muc_jid(self, name):
        # Genera el jid del MUC en función del nombre del recurso.
        return "%s@conference.%s" % (name, self.hostname)

    def cambiar_nombre_server(self):
        ventana_identify = identifyWindow()
        ventana_identify.exec_()

    def ofrecer_ayuda_cliente(self, nickAlumnoSolicitante):
        msg = "El alumno"+nickAlumnoSolicitante+" le solicita ayuda."
        self.enviar_gui_msg(msg,"information")
        self.gui.emit(QtCore.SIGNAL('gestionar_ayuda_cliente'), nickAlumnoSolicitante)


    ## \fnrequest_kick_user(self, nick)
    # \brief Called when the user wants to kick a users frmo the MUC.
    # \param nick Nick of the user to be kicked.
    def expulsar_alumno(self, nick):
        self.log.info(QtGui.QApplication.translate("AppCoreProfe.log", "Expulsando usuario: %1", 
                                                None, QtGui.QApplication.UnicodeUTF8).arg(nick))
        ldebug("Expulsando usuario\n%s\n" % ('-'*80))
        # Not a user anymore.
        del self.users[unicode(nick)]
        self.bot.expulsarAlumno(nick)
        self.detiene_objeto_stream(nick)
        self.gui.emit(QtCore.SIGNAL('comprueba_panelControl'), nick)
        self.gui.emit(QtCore.SIGNAL('comprueba_user_help'), nick)

    ## \fn request_ban_user(self, nick)
    # \brief Called when the user wants to ban a user from the MUC.
    # \param nick Nick of the user to be banned.
    def banear_alumno(self, nick):
        self.log.info(QtGui.QApplication.translate("AppCoreProfe.log", "Prohibiendo acceso al usuario: %1", 
                                                None, QtGui.QApplication.UnicodeUTF8).arg(nick))
        ldebug("Baneando usuario\n%s\n" % ('-'*80))
        # Mark the user as banned, so we can show him in the banned list
        # and maybe unban him later
        self.users[nick]["banned"]=True

        # We need the full JID in order to ban a user, search it.
        jid = self.users[nick].get("jid", None)
        self.bot.bloquearAlumno(jid)
        self.detiene_objeto_stream(nick)
        self.gui.emit(QtCore.SIGNAL('comprueba_panelControl'), nick)
        self.gui.emit(QtCore.SIGNAL('comprueba_user_help'), nick)

    ## \fn request_unban_user(self, nick)
    # \brief Called when the user want to unban a user.
    # \param nick NIck of the user to be unbanned.
    def desbanear_alumno(self, nick):
        self.log.info(QtGui.QApplication.translate("AppCoreProfe.log", "Revocando prohibicion de acceso al usuario: %1", 
                                                None, QtGui.QApplication.UnicodeUTF8).arg(nick))
        ldebug("Desbaneando usuario: '%s'\n%s\n" % (nick, ('-'*80)))
        if nick in self.users.keys(): # Se filtra la posibilidad de que se haya seleccionado un usuario local.
            # We need the full JID to unban the user.
            jid = self.users[nick].get("jid", None)
            if jid:
                self.bot.desbloquearAlumno(jid)
                # We need to delete the user because he is not in the MUC
                # and we don't want him to appear in the connected users list.
                del self.users[nick]

    ## \fn identificar_alumno(self, nick)
    # \brief Requests a user to identify itself.
    # \param nick Nick of the user to identify.
    def identificar_alumno(self, nick, username):
        self.log.info(QtGui.QApplication.translate("AppCoreProfe.log", "Identificando usuario: %1", 
                                                None, QtGui.QApplication.UnicodeUTF8).arg(nick))
        linfo("Identificando usuario '%s'" % nick)
        msg = "IDENTIFY|"+str(username)
        self.sendmessage(nick, msg)

    def habilita_ayuda_user(self, nick):
        msg = "AYUDA-ACTIVA"
        self.sendmessage(nick,msg)

    def habilitar_activa_comp(self):
        msg = "ACTIVA-COMP"
        ipAlumno = ""
        for user in self.users:
            if ipAlumno != self.users[user]["IP"]:
                nick = self.users[user]["nick"]
                self.sendmessage(nick, msg)
    def deshabilitar_activa_comp(self):
        msg = "DESACTIVA-COMP"
        ipAlumno = ""
        for user in self.users:
            if ipAlumno != self.users[user]["IP"]:
                nick = self.users[user]["nick"]
                self.sendmessage(nick, msg)
    ## \fn def sendmessage(self, nick, msg):
    # \brief Funcion que envia mensaje al serverprofesor
    # \param nick Se pasa el nick del usuario al que se le va a enviar el mensaje
    # \param msg Es el mensaje que se le va a enviar al usuario
    def sendmessage(self, nick, msg):
        try:
            if nick in self.users.keys():
                jid = self.users[nick].get("jid", None)
                self.bot.mensajeAlumno(jid, self.jid, msg)
        except Exception, e:
            print e

    ## \fn request_open_muc(self)
    # \brief Se abre un grupo que fue cerrado previamente. Se pueden añadir mas usuarios.
    def abrir_muc(self):
        self.log.info(QtGui.QApplication.translate("AppCoreProfe.log", "Abriendo el grupo."))
        ldebug("Abriendo MUC\n%s\n" % ('-'*80))
        self.bot.abrirMuc()

    ## \fn request_close_muc(self)
    # \brief El usuario servidor decide cerrar el grupo. No se conectarán mas usuarios.
    def cerrar_muc(self):
        self.log.info(QtGui.QApplication.translate("AppCoreProfe.log", "Cerrando el grupo."))
        ldebug("Cerrando MUC\n%s\n" % ('-'*80))
        self.bot.cerrarMuc()

    ## \fn actualiza_objeto_stream(self)
    # \brief Se encarga de gestionar el objeto del sgestion de la emision de video para el cambio de nombre del usuario.
    # \param nick el nick del usuario baneado, expulsado o que se desconecta de la sala
    def detiene_objeto_stream(self, nick=None):
        if nick != None:
            if nick == self.gui.usuario_select_stream:
                self.gui.usuario_select_stream = None
        else:
            self.gui.usuario_select_stream = None
            self.gui.lista_usuarios_stream = False

    ## \fn activa_usuarios_stream(self, alumno=None)
    # \brief Activa los atributos del sgestion para la emision del stream
    # \param alumno, diferencia si es alumno individual o emision en grupo
    def activa_usuarios_stream(self, alumno=None):
        if alumno != None:
            self.gui.usuario_select_stream = alumno
        else:
            self.gui.lista_usuarios_stream = True

    ## \fn borrar_alumno_lista(self, nick):
    # \brief Eliminará a un usuario de la lista
    # \param nick Nick del usuario que se quiere eliminar.
    def borrar_alumno_lista(self, nick):
        self.log.info(QtGui.QApplication.translate("AppCoreProfe.log", "Usuario desconectado: %1", 
                                                None, QtGui.QApplication.UnicodeUTF8).arg(nick))
        ldebug("CORE :: Removing user: '%s'" % nick)
        # Se comprueba si el usuario está en la lista y se procede a su eliminación en caso de que exista.
        if nick in self.users.keys():
            # Un usuario bloqueado no puede ser eliminado de la lista.
            # ya que se encuentra en la lista de usuarios baneados.
            if not self.users[nick]["banned"]:
                del self.users[nick]

        self.detiene_objeto_stream(nick)
        self.gui.emit(QtCore.SIGNAL('update_gui'), self.users)
        self.gui.emit(QtCore.SIGNAL('update_tabla_desktop'))
        self.gui.emit(QtCore.SIGNAL('comprueba_panelControl'), nick)
        self.gui.emit(QtCore.SIGNAL('comprueba_user_help'), nick)

    ## \fn anadir_alumno_lista(self, data)
    # \brief Añade un usuario a la lista
    # \param data Datos del nuevo usuario (nick, ip, jid...)
    def anadir_alumno_lista(self, data):
        self.log.info(QtGui.QApplication.translate("AppCoreProfe.log", "Usuario conectado: %1", 
                                                None, QtGui.QApplication.UnicodeUTF8).arg(data["nick"]))
        ldebug("CORE :: Adding new user: '%s'" % data["nick"])
        # Directly add the data to the dict using the nick as key.
        self.users[data["nick"]] = data
        # The user is not banned (yet :) )
        self.users[data["nick"]]['banned'] = False
        nick = self.users[data["nick"]].get('nick')
        self.bot.ce_handlers["STATUS_SERVERHC"](nick)
        #Atributo que recoge el nick del usuario nuevo
        self.gui.nuevo_user = nick
        self.gui.update_gui(self.users)
        if self.cambio_delay != 5:
            msg = "DELAY|"+str(self.cambio_delay)
            self.sendmessage(nick,msg)

    def cerrarEvento(self):
        self.log.info(QtGui.QApplication.translate("AppCoreProfe.log", "Saliendo de la aplicación...", 
            None, QtGui.QApplication.UnicodeUTF8))
        # Este evento se llama cuando se cierra la ventana del programa.
        # Devuelve True si es correcto continuar. False si se cancela el cierre
        self.flag = True
        self.destruir_recurso()
        self.xmpp_server.stop()
        self.log.info(QtGui.QApplication.translate("AppCoreProfe.log", "Cerrando la aplicación.", 
            None, QtGui.QApplication.UnicodeUTF8))
        return True
    
    #metodo para el envio de mensajes informativos al usuario servidor
    def enviar_gui_msg(self, msg, tipo):
        self.gui.emit(QtCore.SIGNAL("msgReceived"), msg, tipo)

    def stop_progressBar(self):
        self.gui.emit(QtCore.SIGNAL("stopPBAR"))

    def start_progressBar(self, procesoKey, command, nick=None):
        self.gui.emit(QtCore.SIGNAL("ProgressBar"), procesoKey, command, nick)

    ## \fn destruir_recurso(self)
    # \brief Llamado cuando el usuario quiere eliminar el grupo activo.
    def destruir_recurso(self):
        os.system('rm -rf /home/usuario/mnt/HGR/RECURSOS/' + self.muc_name)
        self.log.info(QtGui.QApplication.translate("AppCoreProfe.log", "Eliminando grupo: %1", 
                                            None, QtGui.QApplication.UnicodeUTF8).arg(self.muc_name))
        ldebug("Destroying resource\n%s\n" % ('-'*80))

        if self.bot:
            if self.flag:
                self.bot.ce_handlers["STOP_SERVERHC"]()
            self.mdns.unpublish()
            self.bot.destruirMuc()
            self.bot.disconnect()
            self.sServidor.shutdown(SHUT_RDWR)
            self.sServidor.close()

        else:
            if self.flag:
                self.gui.emit(QtCore.SIGNAL('deshabilitar_HC'))

        self.bot = None
        self.users = dict()
        parar_procesos_activos()
        parar_vlc()
        os.system('rm -rf /tmp/.hgr')
        self.log.info(QtGui.QApplication.translate("AppCoreProfe.log", "Grupo eliminado."))

    ## \fn _setup_logging(self)
    # \brief Sets up the logging class attribute.
    def _setup_logging(self):
        logging.config.fileConfig(LOGGING_CONFIG_FILE)
        self.log = logging.getLogger('hgr.server')

    ## \fn _setup_mdns(sef)
    # \brief Sets up the resource publisher.
    def _setup_mdns(self):
        # Se instancia el publicador de recursos.
        DBusQtMainLoop(set_as_default=True)
        self.mdns = SMdns()

    def view_grupos(self):
        DBusGMainLoop(set_as_default=True)
        self.mdns = SMdns()
        self.mdns.browse(self.handle_resource_found, self.handle_resource_lost)

    def handle_resource_found(self, name, domain, address, port, info_dict):
        info_keys = info_dict.keys()
        if 'muc_jid' in info_keys and 'muc_name' in info_keys and 'prof_name' in info_keys and 'prof_jid' in info_keys:
            info_dict['service_name'] = name
            with self.resource_list_lock:
                self.resource_list[info_dict['muc_jid']] = info_dict
                ADDgrupo = str(info_dict["muc_name"])
                self.listaGrupos.append(ADDgrupo)

    def handle_resource_lost(self, *args):
        resource_index = None
        for resource_key, resource_value in self.resource_list.iteritems():
            if resource_value['service_name'] == args[0]:
                resource_index = resource_key
                continue
        if resource_index is not None:
            resource = self.resource_list[resource_index]
            with self.resource_list_lock:
                del self.resource_list[resource_index]
                DELgrupo = str(resource["muc_name"]) 
                while DELgrupo in self.listaGrupos:
                    self.listaGrupos.remove(DELgrupo) 
 

if __name__=="__main__":

    hga = AppCoreProfe()
    
## \class ProgressBar_thread
# \brief Qthread para controlar la ejecución de la barra de progreso
class StartServer_thread(QThread):
    def __init__(self, xmpp_server):
        QThread.__init__(self)
        self.xmpp_server = xmpp_server
        self.estado = False

    def run(self):
        try:           
            self.xmpp_server.start()
            self.estado = True
        except Exception, e:
            lerror("StartServer_thread (appcoreprofe.py): %s" % e)



## \class HiloSocket
# \brief Qthread encargado de ejecutar en un hilo aparte la apertura del socket para 
# \recibir imagene de escritorios de clientes
class HiloSocket(Thread):
    def __init__(self, core):
        Thread.__init__(self)
        self.core = core

    def run(self):
        while self.core.bot != None:
            self.core.socketImagenes()
